from netgen.csg import *
from math import pi

N = 2
L = 2*pi

DomainVol = L*L*L


from netgen.meshing import *

def GenerateCubeMesh(N,L):

    mesh = Mesh()

    pids = []
    for i in range(N+1):
        for j in range(N+1):
            for k in range(N+1):
                pids.append (mesh.Add(MeshPoint(Pnt(L*i/N, L*j/N, L*k/N))))
                if k is 0:
                    slave = pids[-1]
                if k is N:
                    mesh.AddPointIdentification(pids[-1],slave,identnr=1,type=2)
            
    for i in range(N):
        for j in range(N):
            for k in range(N):
                base = i * (N+1)*(N+1) + j*(N+1) + k
                baseup = base+(N+1)*(N+1)
                pnum = [base,base+1,base+(N+1)+1,base+(N+1),
                        baseup, baseup+1, baseup+(N+1)+1, baseup+(N+1)]
                elpids = [pids[p] for p in pnum]
                mesh.Add(Element3D(1,elpids))

    def AddSurfEls(p1, dx, dy, facenr):
        for i in range(N):
            for j in range(N):
                base = p1 + i*dx+j*dy
                pnum = [base, base+dx, base+dx+dy, base+dy]
                elpids = [pids[p] for p in pnum]
                mesh.Add(Element2D(facenr,elpids))
                        
    fd1 = mesh.Add(FaceDescriptor(surfnr=1,domin=1,bc=1))
    AddSurfEls(0, 1, N+1, facenr=fd1)  

    fd2 = mesh.Add(FaceDescriptor(surfnr=2,domin=1,bc=2))
    AddSurfEls(0, (N+1)*(N+1), 1, facenr=fd2)  

    fd3 = mesh.Add(FaceDescriptor(surfnr=3,domin=1,bc=3))
    AddSurfEls(0, N+1, (N+1)*(N+1), facenr=fd3)    

    fd4 = mesh.Add(FaceDescriptor(surfnr=4,domin=1,bc=4))
    AddSurfEls((N+1)**3-1, -(N+1), -1, facenr=fd4) 

    fd5 = mesh.Add(FaceDescriptor(surfnr=5,domin=1,bc=5))
    AddSurfEls((N+1)**3-1, -(N+1)*(N+1), -(N+1), facenr=fd5)   

    fd6 = mesh.Add(FaceDescriptor(surfnr=6,domin=1,bc=6))
    AddSurfEls((N+1)**3-1, -1, -(N+1)*(N+1), facenr=fd6)

    return mesh

# How is it possible to include the above defined periodicity in the mesh?    

mesh = GenerateCubeMesh(N,L)
mesh.Save("PeriodicCubeMesh.vol")

from ngsolve import *
mesh = Mesh(mesh)
Draw(mesh)
    

fes = Periodic(H1(mesh,dirichlet=[2]))
u,v = fes.TrialFunction(),fes.TestFunction()

a = BilinearForm(fes)
a += SymbolicBFI(grad(u) * grad(v))

f = LinearForm(fes)
f += SymbolicLFI(sin(10*x+10*y)*v)

a.Assemble()
f.Assemble()

u = GridFunction(fes,"u")

u.vec.data = a.mat.Inverse(fes.FreeDofs()) * f.vec

Draw(u)
